import paho.mqtt.client as mqtt
import json
from datetime import datetime

# MQTT Broker 配置
BROKER = "192.168.8.139"  # 替换为你的 MQTT Broker 地址
PORT = 1883
SITE_CODE = "124127"
TOPIC_EVENT = f"HCMS/{SITE_CODE}/S2M/event"
TOPIC_KEEPALIVE_IN = f"HCMS/{SITE_CODE}/S2M/keepalive"
TOPIC_KEEPALIVE_OUT = f"HCMS/{SITE_CODE}/M2S/keepalive"
TOPIC_CONFIRM = f"HCMS/{SITE_CODE}/M2S/confirm"


# 连接回调
def on_connect(client, userdata, flags, rc):
    print(f"Connected to MQTT Broker with result code {rc}")
    client.subscribe([(TOPIC_EVENT, 0), (TOPIC_KEEPALIVE_IN, 0)])
    print(f"Subscribed to {TOPIC_EVENT} and {TOPIC_KEEPALIVE_IN}")


# 消息处理回调
def on_message(client, userdata, msg):
    payload = msg.payload.decode()
    topic = msg.topic
    print(f"Received: {topic} - {payload}")

    if topic == TOPIC_KEEPALIVE_IN:
        # 处理心跳报文
        current_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        client.publish(TOPIC_KEEPALIVE_OUT, current_time, qos=0)
        print(f"Sent keepalive reply: {TOPIC_KEEPALIVE_OUT} - {current_time}")

    elif topic == TOPIC_EVENT:
        try:
            data = json.loads(payload)
            header = data.get("header", {})
            data_body = data.get("dataBody", {})

            function = header.get("function", "unknown")
            index = header.get("index", 0)
            order_sn = data_body.get("orderSn", "unknown")

            # 构造确认报文
            confirm_payload = {
                "header": {
                    "version": "V2.0",
                    "timeStamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                    "index": index,
                    "function": f"{function}Conf"
                },
                "dataBody": {
                    "orderSn": order_sn,
                    "result": 1
                }
            }

            # 发送确认
            client.publish(TOPIC_CONFIRM, json.dumps(confirm_payload), qos=0)
            print(f"Sent confirmation: {TOPIC_CONFIRM} - {json.dumps(confirm_payload)}")

        except json.JSONDecodeError:
            print("Failed to parse JSON payload")
        except Exception as e:
            print(f"Error processing message: {e}")


# 主程序
def main():
    client = mqtt.Client()
    client.on_connect = on_connect
    client.on_message = on_message

    client.connect(BROKER, PORT, 60)
    client.loop_forever()


if __name__ == "__main__":
    main()