import paho.mqtt.client as mqtt
import json
from datetime import datetime

# MQTT Broker 配置
BROKER = "192.168.8.139"  # 替换为你的 MQTT Broker 地址
PORT = 1883
SITE_CODE = "124127"
TOPIC_STATE = f"HCMS/{SITE_CODE}/S2M/state"
TOPIC_KEEPALIVE_IN = f"HCMS/{SITE_CODE}/S2M/keepalive"
TOPIC_KEEPALIVE_OUT = f"HCMS/{SITE_CODE}/M2S/keepalive"
TOPIC_REQUEST = f"HCMS/{SITE_CODE}/M2S/request"


# 连接回调
def on_connect(client, userdata, flags, rc):
    print(f"Connected to MQTT Broker with result code {rc}")
    client.subscribe([(TOPIC_STATE, 0), (TOPIC_KEEPALIVE_IN, 0)])
    print(f"Subscribed to {TOPIC_STATE} and {TOPIC_KEEPALIVE_IN}")


# 消息处理回调
def on_message(client, userdata, msg):
    payload = msg.payload.decode()
    topic = msg.topic
    print(f"Received: {topic} - {payload}")

    if topic == TOPIC_KEEPALIVE_IN:
        # 处理心跳报文
        current_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        client.publish(TOPIC_KEEPALIVE_OUT, current_time, qos=0)
        print(f"Sent keepalive reply: {TOPIC_KEEPALIVE_OUT} - {current_time}")

    elif topic == TOPIC_STATE:
        try:
            data = json.loads(payload)
            header = data.get("header", {})
            function = header.get("function", "unknown")
            timestamp = header.get("timeStamp", "unknown")
            index = header.get("index", 0)
            print(f"State - Function: {function}, TimeStamp: {timestamp}, Index: {index}")
            # 状态类报文无需回复，此处仅记录
        except json.JSONDecodeError:
            print("Failed to parse JSON payload")


# 发送状态召唤请求（可选）
def send_state_call(client, function):
    request_payload = {
        "header": {
            "version": "V2.0",
            "timeStamp": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            "index": 1,  # 可递增管理
            "function": f"{function}Call"
        }
    }
    client.publish(TOPIC_REQUEST, json.dumps(request_payload), qos=0)
    print(f"Sent state call request: {function}Call")


# 主程序
def main():
    client = mqtt.Client()
    client.on_connect = on_connect
    client.on_message = on_message

    client.connect(BROKER, PORT, 60)

    # 示例：发送 swapState 召唤请求（可选）
    # send_state_call(client, "swapState")

    client.loop_forever()


if __name__ == "__main__":
    main()