import paho.mqtt.client as mqtt
import json
import time
import logging

class MQTTClient:
    def __init__(self, host="123.6.102.119", port=1883, username="emqx_test", password="emqx_test"):
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.client = mqtt.Client(client_id="DirectChargingClient", protocol=mqtt.MQTTv311)
        self.client.username_pw_set(self.username, self.password)
        self.client.on_connect = self.on_connect
        self.client.on_disconnect = self.on_disconnect
        self.connected = False

    def on_connect(self, client, userdata, flags, rc):
        if rc == 0:
            logging.info("成功连接到 MQTT 服务器")
            self.connected = True
        else:
            logging.error(f"MQTT 连接失败，错误码: {rc}")
            self.connected = False

    def on_disconnect(self, client, userdata, rc):
        logging.warning("MQTT 断开连接")
        self.connected = False

    def connect(self):
        try:
            self.client.connect(self.host, self.port, 60)
            self.client.loop_start()
            # 等待连接完成
            for _ in range(10):  # 最多等待 5 秒
                if self.connected:
                    return True
                time.sleep(0.5)
            logging.error("MQTT 连接超时")
            return False
        except Exception as e:
            logging.error(f"MQTT 连接失败: {str(e)}")
            return False

    def publish_message(self, message, topic="hejin/charging/log"):
        try:
            if not self.connected:
                logging.error("MQTT client not connected")
                return False
            msg = json.dumps(message)
            result = self.client.publish(topic, msg, qos=1)
            if result.rc == mqtt.MQTT_ERR_SUCCESS:
                logging.info(f"发布消息成功: {msg}")
                return True
            else:
                logging.error(f"发布消息失败，错误码: {result.rc}")
                return False
        except Exception as e:
            logging.error(f"MQTT 发布消息失败: {str(e)}")
            return False

    def disconnect(self):
        try:
            self.client.loop_stop()
            self.client.disconnect()
            self.connected = False
            logging.info("MQTT 断开连接")
        except Exception as e:
            logging.error(f"MQTT 断开连接失败: {str(e)}")