import taosrest
import psycopg2
from datetime import datetime
import binascii
import logging
import time

# 配置日志
logging.basicConfig(
    filename='charge_gun_state_record.log',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    encoding='utf-8'
)

class ChargeGunStateMigrator:
    def __init__(self):
        # TDengine连接参数
        self.tdengine_config = {
            'host': '123.6.102.119',
            'port': 6041,
            'user': 'readonly_user',
            'password': 'Aassword123',
            'database': 'antsev'
        }

        # PostgreSQL连接参数
        self.pg_config = {
            'host': '123.6.102.119',
            'port': 5432,
            'database': 'tms-design',
            'user': 'postgres',
            'password': '687315e66ae24eeab8bb5c0441a40d79'
        }

        self.td_conn = None
        self.td_cursor = None
        self.pg_conn = None
        self.pg_cursor = None
        self.last_processed_ts = None
        self.processed_connectors = set()

    def connect(self):
        """建立与两个数据库的连接"""
        max_retries = 3
        retry_delay = 10  # 秒
        for attempt in range(max_retries):
            try:
                # 连接到TDengine
                logging.info(f"尝试连接到TDengine (第 {attempt + 1} 次): {self.tdengine_config}")
                rest_url = f"http://{self.tdengine_config['host']}:{self.tdengine_config['port']}"
                self.td_conn = taosrest.connect(
                    url=rest_url,
                    user=self.tdengine_config['user'],
                    password=self.tdengine_config['password'],
                    database=self.tdengine_config['database']
                )
                self.td_cursor = self.td_conn.cursor()
                logging.info("成功连接到TDengine")

                # 测试查询以验证连接
                self.td_cursor.execute("SELECT SERVER_VERSION()")
                version = self.td_cursor.fetchone()
                logging.info(f"TDengine 服务器版本: {version[0]}")

                # 连接到PostgreSQL
                logging.info(f"尝试连接到PostgreSQL: {self.pg_config}")
                self.pg_conn = psycopg2.connect(
                    host=self.pg_config['host'],
                    port=self.pg_config['port'],
                    database=self.pg_config['database'],
                    user=self.pg_config['user'],
                    password=self.pg_config['password']
                )
                self.pg_conn.autocommit = True
                self.pg_cursor = self.pg_conn.cursor()
                logging.info("成功连接到PostgreSQL")
                break  # 连接成功，退出重试循环

            except Exception as e:
                logging.error(f"连接错误 (第 {attempt + 1} 次): {str(e)}")
                if attempt < max_retries - 1:
                    logging.info(f"将在 {retry_delay} 秒后重试...")
                    time.sleep(retry_delay)
                else:
                    raise

    def parse_hex_data(self, hex_data):
        """根据协议解析十六进制数据"""
        try:
            # 移除空格并将十六进制字符串转换为字节
            hex_bytes = bytes.fromhex(hex_data.replace(" ", ""))

            # 验证帧起始（应该是"JX"）
            if hex_bytes[0:2] != b'JX':
                return None

            # 提取命令
            command = hex_bytes[2:3].hex().upper()

            # 提取枪号（假设在协议中枪号位于第11字节，0x01表示A枪，0x02表示B枪）
            connector_suffix = '1' if hex_bytes[11] == 0x01 else '2'

            # 初始化数据字典
            data = {
                'command': command,
                'connector_suffix': connector_suffix,
                'status': 0,  # 默认离网
                'park_status': 10,  # 默认空闲
                'lock_status': 10  # 默认已解锁
            }

            # 25H - 充电信息
            if command == '25':
                data.update({
                    'status': 3  # 充电中
                })

            # 23H - 最新充电订单
            elif command == '23':
                data.update({
                    'status': 1  # 充电完成，枪状态为空闲
                })

            return data

        except Exception as e:
            logging.error(f"解析十六进制数据时出错: {str(e)}")
            return None

    def migrate_data(self):
        """将新数据从TDengine迁移到PostgreSQL的charge_gun_state_record表"""
        while True:
            try:
                # 如果last_processed_ts为空，初始化为当前时间
                if self.last_processed_ts is None:
                    try:
                        # 避免使用 MAX(ts)，改用 ORDER BY ts DESC LIMIT 1 获取最新时间戳
                        self.td_cursor.execute("SELECT ts FROM antsev.charge_jiuxing ORDER BY ts DESC LIMIT 1")
                        result = self.td_cursor.fetchone()
                        self.last_processed_ts = result[0] if result and result[0] else datetime.now()
                    except Exception as e:
                        logging.error(f"获取最新时间戳失败: {str(e)}，使用当前时间作为默认值")
                        self.last_processed_ts = datetime.now()
                    logging.info(f"初始化last_processed_ts: {self.last_processed_ts}")

                # 查询新数据
                query = f"SELECT * FROM antsev.charge_jiuxing WHERE ts > '{self.last_processed_ts}' ORDER BY ts"
                self.td_cursor.execute(query)
                rows = self.td_cursor.fetchall()

                if not rows:
                    logging.info("没有新数据，休眠10秒")
                    time.sleep(10)
                    continue

                for row in rows:
                    try:
                        # 从TDengine行中提取数据
                        timestamp = row[0]  # 时间戳
                        pile_id = row[3]    # 充电桩ID
                        hex_data = row[12]  # 十六进制数据

                        # 记录原始数据
                        logging.info(f"处理记录: ts={timestamp}, pile_id={pile_id}, hex_data={hex_data}")

                        # 解析十六进制数据
                        parsed_data = self.parse_hex_data(hex_data)
                        if not parsed_data:
                            logging.warning(f"无法解析 hex_data: {hex_data}，跳过此记录")
                            continue

                        # 构造connector_id（pile_id + 枪号后缀）
                        connector_id = f"{pile_id}{parsed_data['connector_suffix']}"

                        # 构造唯一标识（connector_id + timestamp）
                        connector_key = f"{connector_id}_{timestamp}"

                        # 检查记录是否已存在
                        check_query = """
                        SELECT 1 FROM charge_gun_state_record WHERE connector_id = %s AND created_at = %s
                        """
                        self.pg_cursor.execute(check_query, (connector_id, timestamp))
                        exists = self.pg_cursor.fetchone() is not None

                        # 如果记录已存在，跳过
                        if exists or connector_key in self.processed_connectors:
                            logging.info(f"充电枪 {connector_id} 的状态记录已存在，时间戳 {timestamp}，跳过")
                            continue

                        # 准备插入PostgreSQL的数据
                        insert_query = """
                        INSERT INTO public.charge_gun_state_record (
                            connector_id, status, park_status, lock_status, created_at
                        ) VALUES (%s, %s, %s, %s, %s)
                        """
                        values = (
                            connector_id,
                            parsed_data['status'],
                            parsed_data['park_status'],
                            parsed_data['lock_status'],
                            timestamp
                        )

                        self.pg_cursor.execute(insert_query, values)
                        self.processed_connectors.add(connector_key)
                        logging.info(f"插入充电枪 {connector_id} 的状态记录，时间戳 {timestamp}")

                        # 记录插入的完整数据
                        log_values = {
                            'connector_id': connector_id,
                            'status': parsed_data['status'],
                            'park_status': parsed_data['park_status'],
                            'lock_status': parsed_data['lock_status'],
                            'created_at': timestamp
                        }
                        logging.info(f"插入到 charge_gun_state_record 表的数据: {log_values}")

                        # 更新last_processed_ts
                        self.last_processed_ts = max(self.last_processed_ts, timestamp)

                    except Exception as e:
                        logging.error(f"处理时间戳为 {timestamp} 的记录时出错: {str(e)}")
                        continue

            except Exception as e:
                logging.error(f"迁移过程中出错: {str(e)}")
                time.sleep(10)  # 出错后休眠10秒后重试

    def close(self):
        """关闭数据库连接"""
        try:
            if self.td_cursor:
                self.td_cursor.close()
            if self.td_conn:
                self.td_conn.close()
            if self.pg_cursor:
                self.pg_cursor.close()
            if self.pg_conn:
                self.pg_conn.close()
            logging.info("数据库连接已关闭")
        except Exception as e:
            logging.error(f"关闭连接时出错: {str(e)}")
            raise

    def run(self):
        """运行迁移的主方法"""
        try:
            self.connect()
            self.migrate_data()
        except Exception as e:
            logging.error(f"迁移失败: {str(e)}")
            raise
        finally:
            self.close()

if __name__ == "__main__":
    migrator = ChargeGunStateMigrator()
    migrator.run()