import taosrest
import psycopg2
from datetime import datetime
import binascii
import logging
import uuid
import time

# 配置日志
logging.basicConfig(
    filename='charge_order_info.log',
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    encoding='utf-8'
)

class ChargeOrderInfoMigrator:
    def __init__(self):
        # TDengine连接参数
        self.tdengine_config = {
            'host': '123.6.102.119',
            'port': 6041,
            'user': 'readonly_user',
            'password': 'Aassword123',
            'database': 'antsev'
        }

        # PostgreSQL连接参数
        self.pg_config = {
            'host': '123.6.102.119',
            'port': 5432,
            'database': 'tms-design',
            'user': 'postgres',
            'password': '687315e66ae24eeab8bb5c0441a40d79'
        }

        self.td_conn = None
        self.td_cursor = None
        self.pg_conn = None
        self.pg_cursor = None
        self.last_processed_ts = None
        self.processed_uuids = set()

    def connect(self):
        """建立与两个数据库的连接"""
        max_retries = 3
        retry_delay = 10  # 秒
        for attempt in range(max_retries):
            try:
                # 连接到TDengine
                logging.info(f"尝试连接到TDengine (第 {attempt + 1} 次): {self.tdengine_config}")
                rest_url = f"http://{self.tdengine_config['host']}:{self.tdengine_config['port']}"
                self.td_conn = taosrest.connect(
                    url=rest_url,
                    user=self.tdengine_config['user'],
                    password=self.tdengine_config['password'],
                    database=self.tdengine_config['database']
                )
                self.td_cursor = self.td_conn.cursor()
                logging.info("成功连接到TDengine")

                # 测试查询以验证连接
                self.td_cursor.execute("SELECT SERVER_VERSION()")
                version = self.td_cursor.fetchone()
                logging.info(f"TDengine 服务器版本: {version[0]}")

                # 连接到PostgreSQL
                logging.info(f"尝试连接到PostgreSQL: {self.pg_config}")
                self.pg_conn = psycopg2.connect(
                    host=self.pg_config['host'],
                    port=self.pg_config['port'],
                    database=self.pg_config['database'],
                    user=self.pg_config['user'],
                    password=self.pg_config['password']
                )
                self.pg_conn.autocommit = True
                self.pg_cursor = self.pg_conn.cursor()
                logging.info("成功连接到PostgreSQL")
                break  # 连接成功，退出重试循环

            except Exception as e:
                logging.error(f"连接错误 (第 {attempt + 1} 次): {str(e)}")
                if attempt < max_retries - 1:
                    logging.info(f"将在 {retry_delay} 秒后重试...")
                    time.sleep(retry_delay)
                else:
                    raise

    def parse_bcd_time(self, bcd_bytes):
        """解析BCD码时间（假设格式为 YYMMDDHHMMSS）"""
        try:
            year = 2000 + (bcd_bytes[0] >> 4) * 10 + (bcd_bytes[0] & 0x0F)
            month = (bcd_bytes[1] >> 4) * 10 + (bcd_bytes[1] & 0x0F)
            day = (bcd_bytes[2] >> 4) * 10 + (bcd_bytes[2] & 0x0F)
            hour = (bcd_bytes[3] >> 4) * 10 + (bcd_bytes[3] & 0x0F)
            minute = (bcd_bytes[4] >> 4) * 10 + (bcd_bytes[4] & 0x0F)
            second = (bcd_bytes[5] >> 4) * 10 + (bcd_bytes[5] & 0x0F)
            return datetime(year, month, day, hour, minute, second)
        except Exception as e:
            logging.error(f"解析BCD时间出错: {str(e)}")
            return None

    def parse_hex_data(self, hex_data, timestamp, pile_id):
        """根据协议解析十六进制数据"""
        try:
            # 移除空格并将十六进制字符串转换为字节
            hex_bytes = bytes.fromhex(hex_data.replace(" ", ""))

            # 验证帧起始（应该是"JX"）
            if hex_bytes[0:2] != b'JX':
                return None

            # 提取命令
            command = hex_bytes[2:3].hex().upper()

            # 提取枪号（假设在协议中枪号位于第11字节，0x01表示A枪，0x02表示B枪）
            connector_suffix = '1' if hex_bytes[11] == 0x01 else '2'

            # 构造connector_id
            connector_id = f"{pile_id}{connector_suffix}"

            # 初始化数据字典
            data = {
                'command': command,
                'connector_id': connector_id,
                'start_time': timestamp,
                'end_time': timestamp,
                'total_power': 0.0,
                'elec_money': 0.0,
                'sevice_money': 0.0,
                'total_money': 0.0,
                'stop_reason': 0,
                'sum_period': 1,
                'ident_code': None,
                'charge_status': 1,
                'fail_reason': 0,
                'operator_id': 'K1TUBMOLH',
                'vin': None,
                'compute_start_time': timestamp,
                'compute_end_time': timestamp,
                'compute_total_power': 0.0,
                'compute_total_elec_money': 0.0,
                'compute_total_sevice_money': 0.0,
                'compute_total_money': 0.0,
                'compute_discount_total_money': 0.0,
                'compute_discount_total_elec_money': 0.0,
                'compute_discount_total_sevice_money': 0.0,
                'org_code': 'MACPWMWG69',
                'merchant_id': '1863384914068400094',
                'start_soc': 0.0,
                'end_soc': 0.0,
                'order_amount': 0.0
            }

            # 25H - 充电信息
            if command == '25':
                data.update({
                    'charge_status': 2
                })

            # 23H - 最新充电订单
            elif command == '23':
                # 起始充电电量（字节119-122，分辨率0.01kWh）
                start_power = int.from_bytes(hex_bytes[119:123], byteorder='little') * 0.01
                # 结束充电电量（字节123-126，分辨率0.01kWh）
                end_power = int.from_bytes(hex_bytes[123:127], byteorder='little') * 0.01
                # 计算总电量
                total_power = end_power - start_power

                # 充电金额（电费+服务费+停车费）
                electricity_fee = int.from_bytes(hex_bytes[147:151], byteorder='little') * 0.01
                service_fee = int.from_bytes(hex_bytes[151:155], byteorder='little') * 0.01
                parking_fee = int.from_bytes(hex_bytes[155:159], byteorder='little') * 0.01
                total_money = electricity_fee + service_fee + parking_fee

                # 充电开始时间和结束时间（字节127-132和133-138，格式为BCD码）
                start_time = self.parse_bcd_time(hex_bytes[127:133])
                end_time = self.parse_bcd_time(hex_bytes[133:139])

                # 假设SOC（协议中可能有相关字段，需确认）
                start_soc = float(int.from_bytes(hex_bytes[139:141], byteorder='little')) if len(
                    hex_bytes) > 140 else 0.0
                end_soc = float(int.from_bytes(hex_bytes[141:143], byteorder='little')) if len(hex_bytes) > 142 else 0.0

                # 假设VIN（协议中可能有相关字段，需确认）
                vin = hex_bytes[143:160].decode('ascii', errors='ignore') if len(hex_bytes) > 159 else None

                data.update({
                    'start_time': start_time if start_time else timestamp,
                    'end_time': end_time if end_time else timestamp,
                    'total_power': total_power,
                    'elec_money': electricity_fee,
                    'sevice_money': service_fee,
                    'total_money': total_money,
                    'charge_status': 4,
                    'compute_start_time': start_time if start_time else timestamp,
                    'compute_end_time': end_time if end_time else timestamp,
                    'compute_total_power': total_power,
                    'compute_total_elec_money': electricity_fee,
                    'compute_total_sevice_money': service_fee,
                    'compute_total_money': total_money,
                    'start_soc': start_soc,
                    'end_soc': end_soc,
                    'vin': vin,
                    'order_amount': total_money
                })

            return data

        except Exception as e:
            logging.error(f"解析十六进制数据时出错: {str(e)}")
            return None

    def migrate_data(self):
        """将新数据从TDengine迁移到PostgreSQL的charge_order_info表"""
        while True:
            try:
                # 如果last_processed_ts为空，初始化为当前时间
                if self.last_processed_ts is None:
                    try:
                        # 避免使用 MAX(ts)，改用 ORDER BY ts DESC LIMIT 1 获取最新时间戳
                        self.td_cursor.execute("SELECT ts FROM antsev.charge_jiuxing ORDER BY ts DESC LIMIT 1")
                        result = self.td_cursor.fetchone()
                        self.last_processed_ts = result[0] if result and result[0] else datetime.now()
                    except Exception as e:
                        logging.error(f"获取最新时间戳失败: {str(e)}，使用当前时间作为默认值")
                        self.last_processed_ts = datetime.now()
                    logging.info(f"初始化last_processed_ts: {self.last_processed_ts}")

                # 查询新数据
                query = f"SELECT * FROM antsev.charge_jiuxing WHERE ts > '{self.last_processed_ts}' ORDER BY ts"
                self.td_cursor.execute(query)
                rows = self.td_cursor.fetchall()

                if not rows:
                    logging.info("没有新数据，休眠10秒")
                    time.sleep(10)
                    continue

                for row in rows:
                    try:
                        # 从TDengine行中提取数据
                        timestamp = row[0]  # 时间戳
                        pile_id = row[3]  # 充电桩ID
                        hex_data = row[12]  # 十六进制数据

                        # 记录原始数据
                        logging.info(f"处理记录: ts={timestamp}, pile_id={pile_id}, hex_data={hex_data}")

                        # 解析十六进制数据
                        parsed_data = self.parse_hex_data(hex_data, timestamp, pile_id)
                        if not parsed_data:
                            logging.warning(f"无法解析 hex_data: {hex_data}，跳过此记录")
                            continue

                        # 生成唯一uuid
                        record_uuid = str(uuid.uuid4()).replace('-', '')

                        # 检查记录是否已存在
                        check_query = """
                        SELECT 1 FROM charge_order_info WHERE uuid = %s
                        """
                        self.pg_cursor.execute(check_query, (record_uuid,))
                        exists = self.pg_cursor.fetchone() is not None

                        # 如果记录已存在，跳过
                        if exists or record_uuid in self.processed_uuids:
                            logging.info(f"订单信息记录已存在，UUID: {record_uuid}，跳过")
                            continue

                        # 准备插入PostgreSQL的数据
                        insert_query = """
                        INSERT INTO public.charge_order_info (
                            uuid, connector_id, start_time, end_time, total_power,
                            elec_money, sevice_money, total_money, stop_reason, sum_period,
                            ident_code, created_at, updated_at, charge_status, fail_reason,
                            operator_id, vin, compute_start_time, compute_end_time,
                            compute_total_power, compute_total_elec_money, compute_total_sevice_money,
                            compute_total_money, compute_discount_total_money,
                            compute_discount_total_elec_money, compute_discount_total_sevice_money,
                            org_code, merchant_id, start_soc, end_soc, order_amount
                        ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
                        """
                        values = (
                            record_uuid,
                            parsed_data['connector_id'],
                            parsed_data['start_time'],
                            parsed_data['end_time'],
                            parsed_data['total_power'],
                            parsed_data['elec_money'],
                            parsed_data['sevice_money'],
                            parsed_data['total_money'],
                            parsed_data['stop_reason'],
                            parsed_data['sum_period'],
                            parsed_data['ident_code'],
                            timestamp,
                            timestamp,
                            parsed_data['charge_status'],
                            parsed_data['fail_reason'],
                            parsed_data['operator_id'],
                            parsed_data['vin'],
                            parsed_data['compute_start_time'],
                            parsed_data['compute_end_time'],
                            parsed_data['compute_total_power'],
                            parsed_data['compute_total_elec_money'],
                            parsed_data['compute_total_sevice_money'],
                            parsed_data['compute_total_money'],
                            parsed_data['compute_discount_total_money'],
                            parsed_data['compute_discount_total_elec_money'],
                            parsed_data['compute_discount_total_sevice_money'],
                            parsed_data['org_code'],
                            parsed_data['merchant_id'],
                            parsed_data['start_soc'],
                            parsed_data['end_soc'],
                            parsed_data['order_amount']
                        )

                        self.pg_cursor.execute(insert_query, values)
                        self.processed_uuids.add(record_uuid)
                        logging.info(f"插入订单信息，UUID: {record_uuid}, 充电枪: {parsed_data['connector_id']}")

                        # 记录插入的完整数据
                        log_values = {
                            'uuid': record_uuid,
                            'connector_id': parsed_data['connector_id'],
                            'start_time': parsed_data['start_time'],
                            'end_time': parsed_data['end_time'],
                            'total_power': parsed_data['total_power'],
                            'elec_money': parsed_data['elec_money'],
                            'sevice_money': parsed_data['sevice_money'],
                            'total_money': parsed_data['total_money'],
                            'stop_reason': parsed_data['stop_reason'],
                            'sum_period': parsed_data['sum_period'],
                            'ident_code': parsed_data['ident_code'],
                            'created_at': timestamp,
                            'updated_at': timestamp,
                            'charge_status': parsed_data['charge_status'],
                            'fail_reason': parsed_data['fail_reason'],
                            'operator_id': parsed_data['operator_id'],
                            'vin': parsed_data['vin'],
                            'compute_start_time': parsed_data['compute_start_time'],
                            'compute_end_time': parsed_data['compute_end_time'],
                            'compute_total_power': parsed_data['compute_total_power'],
                            'compute_total_elec_money': parsed_data['compute_total_elec_money'],
                            'compute_total_sevice_money': parsed_data['compute_total_sevice_money'],
                            'compute_total_money': parsed_data['compute_total_money'],
                            'compute_discount_total_money': parsed_data['compute_discount_total_money'],
                            'compute_discount_total_elec_money': parsed_data['compute_discount_total_elec_money'],
                            'compute_discount_total_sevice_money': parsed_data['compute_discount_total_sevice_money'],
                            'org_code': parsed_data['org_code'],
                            'merchant_id': parsed_data['merchant_id'],
                            'start_soc': parsed_data['start_soc'],
                            'end_soc': parsed_data['end_soc'],
                            'order_amount': parsed_data['order_amount']
                        }
                        logging.info(f"插入到 charge_order_info 表的数据: {log_values}")

                        # 更新last_processed_ts
                        self.last_processed_ts = max(self.last_processed_ts, timestamp)

                    except Exception as e:
                        logging.error(f"处理时间戳为 {timestamp} 的记录时出错: {str(e)}")
                        continue

            except Exception as e:
                logging.error(f"迁移过程中出错: {str(e)}")
                time.sleep(10)  # 出错后休眠10秒后重试

    def close(self):
        """关闭数据库连接"""
        try:
            if self.td_cursor:
                self.td_cursor.close()
            if self.td_conn:
                self.td_conn.close()
            if self.pg_cursor:
                self.pg_cursor.close()
            if self.pg_conn:
                self.pg_conn.close()
            logging.info("数据库连接已关闭")
        except Exception as e:
            logging.error(f"关闭连接时出错: {str(e)}")
            raise

    def run(self):
        """运行迁移的主方法"""
        try:
            self.connect()
            self.migrate_data()
        except Exception as e:
            logging.error(f"迁移失败: {str(e)}")
            raise
        finally:
            self.close()

if __name__ == "__main__":
    migrator = ChargeOrderInfoMigrator()
    migrator.run()